﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Microsoft.Practices.Prism.Commands;
using Microsoft.Practices.Prism.ViewModel;

namespace MVVM_sample
{

    class MemberRegistViewModel : NotificationObject
    {
        #region モデル
        private MemberList m_Model;
        #endregion

        #region 画面構成の属性
        /// <summary>
        /// お名前
        /// </summary>
        private string m_Name;
        public string Name
        {
            get { return m_Name; }
            set
            {
                m_Name = value;
                RaisePropertyChanged("Name");
            }
        }

        /// <summary>
        /// 年齢
        /// </summary>
        private string m_Age;
        public string Age
        {
            get { return m_Age; }
            set
            {
                m_Age = value;
                RaisePropertyChanged("Äge");
            }
        }

        /// <summary>
        /// 性別
        /// </summary>
        private Sex m_Sex;
        public Sex Sex
        {
            get { return m_Sex; }
            set
            {
                m_Sex = value;
                RaisePropertyChanged("Sex");
            }
        }

        /// <summary>
        /// 会員名前リスト
        /// </summary>
        private ObservableCollection<string> m_MemberNameList;
        public ObservableCollection<string> MemberNameList
        {
            get { return m_MemberNameList; }
        }

        /// <summary>
        /// 選択された会員名
        /// </summary>
        private string m_SelectedName;
        public string SelectedName
        {
            get { return m_SelectedName; }
            set
            {
                m_SelectedName = value;
                UpdateSelectedUser();
            }
        }

        private string m_SelectedMemberInfo;
        public string SelectedMemberInfo
        {
            get { return m_SelectedMemberInfo; }
            set
            {
                m_SelectedMemberInfo = value;
                RaisePropertyChanged("SelectedMemberInfo");
            }
        }
        #endregion

        /// <summary>
        /// 選択ユーザーの変更
        /// </summary>
        private void UpdateSelectedUser()
        {
            SelectedMemberInfo = string.Format(
                "{0}, {1}, {2}",
                m_SelectedName,
                m_Model.GetAge(m_SelectedName).ToString(),
                m_Model.GetSex(m_SelectedName).ToString());
        }


        #region コマンド定義
        private DelegateCommand m_RegistCom;
        public DelegateCommand RegistCom
        {
            get { return m_RegistCom; }
        }
        #endregion

        #region コマンド実装
        private void RegistComExec()
        {
            if ((m_Name != null) && (m_Age != null))
            {
                m_Model.Add(
                    new MemberInfo()
                {
                    Name = m_Name,
                    Age = Int32.Parse(m_Age),
                    Sex = m_Sex
                });
            }
        }
        #endregion

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public MemberRegistViewModel()
        {
            // モデルの初期化
            m_Model = new MemberList();

            // モデルの状態変化のノーティファイ通知の受信設定
            m_Model.MemberInfoList.CollectionChanged += MemberInfoList_CollectionChanged;
            
            // 属性の初期化
            m_MemberNameList = new ObservableCollection<string>();
            m_Sex = Sex.MALE;

            // コマンドの初期化
            m_RegistCom = new DelegateCommand(RegistComExec);
        }

        /// <summary>
        /// モデルのリストが変化したことをキャッチするイベントハンドラ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void MemberInfoList_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            switch( e.Action)
            {
                case System.Collections.Specialized.NotifyCollectionChangedAction.Add:
                    foreach (var tgt in e.NewItems)
                    {
                        MemberInfo addObj = tgt as MemberInfo;
                        if (addObj != null)
                        {
                            m_MemberNameList.Add(addObj.Name);
                        }
                    }
                    break;

                case System.Collections.Specialized.NotifyCollectionChangedAction.Remove:
                    foreach (var tgt in e.OldItems)
                    {
                        MemberInfo removeObj = tgt as MemberInfo;
                        if (removeObj != null)
                        {
                            m_MemberNameList.Remove(removeObj.Name);
                        }
                    }
                    break;

                default:
                    break;   // Add/Remove以外は何もしない
            }
        }
    }
}
